/*!
FullCalendar ScrollGrid Plugin v7.0.0-beta.3
Docs & License: https://fullcalendar.io/docs/premium
(c) 2024 Adam Shaw
*/
FullCalendar.ScrollGrid = (function (exports, core, premiumCommonPlugin, internal$1) {
    'use strict';

    function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

    var premiumCommonPlugin__default = /*#__PURE__*/_interopDefault(premiumCommonPlugin);

    /*
    Fires:
    - scrollEnd: (x, y) => void
    */
    class ScrollerSyncer {
        constructor(isHorizontal = false) {
            this.isHorizontal = isHorizontal;
            this.emitter = new internal$1.Emitter();
            this.scrollers = [];
            this.destroyFuncs = [];
            this.isPaused = false;
        }
        handleChildren(scrollers) {
            if (!internal$1.isArraysEqual(this.scrollers, scrollers)) {
                this.destroy();
                for (const scroller of scrollers) {
                    if (scroller) { // could be null
                        this.destroyFuncs.push(this.bindScroller(scroller));
                        this.scrollers.push(scroller);
                    }
                }
            }
        }
        destroy() {
            for (let destroyFunc of this.destroyFuncs) {
                destroyFunc();
            }
            this.destroyFuncs = [];
            this.scrollers = [];
        }
        get x() {
            const { scrollers, masterScroller } = this;
            return (masterScroller || scrollers[0]).x;
        }
        get y() {
            const { scrollers, masterScroller } = this;
            return (masterScroller || scrollers[0]).y;
        }
        scrollTo(scrollArg) {
            this.isPaused = true;
            const { scrollers } = this;
            for (let scroller of scrollers) {
                scroller.scrollTo(scrollArg);
            }
            this.isPaused = false;
        }
        addScrollEndListener(handler) {
            this.emitter.on('scrollEnd', handler);
        }
        removeScrollEndListener(handler) {
            this.emitter.off('scrollEnd', handler);
        }
        bindScroller(scroller) {
            let { isHorizontal } = this;
            const onScroll = (isWheel, isTouch) => {
                if (!this.isPaused) {
                    if (!this.masterScroller || (this.masterScroller !== scroller && (isWheel || isTouch))) {
                        this.assignMaster(scroller);
                    }
                    if (this.masterScroller === scroller) { // dealing with current
                        for (let otherScroller of this.scrollers) {
                            if (otherScroller !== scroller) {
                                if (isHorizontal) {
                                    // TODO: user raw scrollLeft for better performance. No normalization necessary
                                    otherScroller.scrollTo({ x: scroller.x });
                                }
                                else {
                                    otherScroller.scrollTo({ y: scroller.y });
                                }
                            }
                        }
                    }
                }
            };
            const onScrollEnd = () => {
                if (this.masterScroller === scroller) {
                    this.masterScroller = null;
                    this.emitter.trigger('scrollEnd', this.x, this.y);
                }
            };
            scroller.listener.emitter.on('scroll', onScroll);
            scroller.listener.emitter.on('scrollEnd', onScrollEnd);
            return () => {
                scroller.listener.emitter.off('scroll', onScroll);
                scroller.listener.emitter.off('scrollEnd', onScrollEnd);
            };
        }
        assignMaster(masterScroller) {
            this.masterScroller = masterScroller;
            for (let scroller of this.scrollers) {
                if (scroller !== masterScroller) {
                    scroller.endScroll(); // to prevent residual scrolls from reclaiming master
                }
            }
        }
    }

    var plugin = core.createPlugin({
        name: '@fullcalendar/scrollgrid',
        premiumReleaseDate: '2024-12-19',
        deps: [premiumCommonPlugin__default["default"]],
        scrollerSyncerClass: ScrollerSyncer
    });

    // new

    var internal = {
        __proto__: null,
        ScrollerSyncer: ScrollerSyncer
    };

    core.globalPlugins.push(plugin);

    exports.Internal = internal;
    exports["default"] = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, FullCalendar.PremiumCommon, FullCalendar.Internal);
